import datetime
import random
import socket

__author__ = 'jacobshutzman1'

""" This is the server portion of the socket. The protocol specifies
    that it receives te first message as a 'hand-shake', to which it
    reacts with the type of services it provides: It looks for a 4
    letter request asfollows:

    NAME - sends its name,
    TIME - sends the currect time as hh:mm:ss,
    RAND - sends a random integer between 1 and 10.
    EXIT - it quits (closes the connection)
"""
IP = '0.0.0.0'
PORT = 1729
TIME_PRE = '0'


def time_response(now):
    """ Format the time nicely """

    if now.hour < 10:
        rhour = TIME_PRE + str(now.hour)
    else:
        rhour = str(now.hour)
    if now.minute < 10:
        rminute = TIME_PRE + str(now.minute)
    else:
        rminute = str(now.minute)
    if now.second < 10:
        rsecond = TIME_PRE + str(now.second)
    else:
        rsecond = str(now.second)
    response = rhour + ':' + rminute + ':' + rsecond
    return response


def format_response(response):
    """ The first 4 bytes of the response is its length. so the client
        knows how  many bytes are left for the request.
    """
    m_length = str(len(response))
    m_length = m_length.zfill(4)
    message_back = m_length + response
    return message_back


def main():
    """
    Gets the first message as a hand-shake, and then loops
    until EXIT is requested, for each request prepares and
    sends the correct response, including a note on invalid
    input.
    """
    server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server.bind((IP, PORT))
    server.listen(1)
    client, (ip, port) = server.accept()
    data = client.recv(100)             # in case we want a long handshake
    first_response = 'Your options are: TIME, RAND(1-10), NAME or EXIT'
    client.send(first_response)
    '''
    The client is responsible for sending only 4 bytes. We could let it
    send more and indicate back invalid data (see Bad data... message)
    '''
    data = client.recv(4)
    while True:
        if data == 'TIME':
            now = datetime.datetime.now()
            response = time_response(now)
        elif data == 'RAND':
            response = str(random.randint(1, 10))
        elif data == 'NAME':
            response = 'My name is: HAL 9000'
        elif data == 'EXIT':
            break
        else:
            response = 'Bad data, only: TIME, RAND, NAME or EXIT are allowed'
        message_back = format_response(response)
        client.send(message_back)
        data = client.recv(4)
    client.close()
    server.close()
if __name__ == '__main__':
    main()
