import socket
import time
import sys
__author__ = 'jacob shutzman'
myserver = ''
SET_LENGTH = 1024
OK = 200
NOT_FOUND = 404
FIRST_ELEMENT = 0
SECOND_ELEMENT = 1
THIRD_ELEMENT = 2

class Server:
    """ Class for creating HTTP server objects."""
    def __init__(self, port=80):
        """ Constructor """
        self.ip = ''
        self.port = port
        self.www_dir = 'webroot'  # root directory for webpages


    def activate_server(self):
        """ Attempts to open a socket and launch the server """
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:  # For privilege reasons  port 80 may be unavailable (<1024)
            print "Launching HTTP server on ", self.ip, ":", self.port
            self.socket.bind((self.ip, self.port))
        except Exception as e:
            print "Warning: Unavailable port:", self.port, "\n"
            print "Trying a higher port"
            # save the original port for reporting purpose
            user_port = self.port
            self.port = 8080
            try:
                print "Launching HTTP server on ", self.ip, ":", self.port
                self.socket.bind((self.ip, self.port))
            except Exception as e:
                print "ERROR: Failed to open sockets for ports ", user_port,\
                      " and 8080. "
                print "Try running the Server in a privileged user mode."
                self.shutdown()
                sys.exit(1)
        print "Server opened the socket with port:", self.port
        print "Press Ctrl+C to shut down the server and exit."
        self._wait_for_connections()


    def shutdown(self):
        """ Shut down the server """
        try:
            print "Shutting down the server"
            myserver.socket.shutdown(socket.SHUT_RDWR)
        except Exception as e:
            print "Warning: could not shut down the socket. " \
                  "Perhaps already closed?", e


    def _attach_headers(self, code, file_type, length=0):
        """ Generates HTTP response Headers"""
        # determine response code
        header = ''
        if (code == OK):
            header = 'HTTP/1.1 200 OK\n'
        elif (code == NOT_FOUND):
            header = 'HTTP/1.1 404 Not Found\n'
        # write more headers later
        current_date = time.strftime("%a, %d %b %Y %H:%M:%S",
                                     time.localtime())
        header += 'Date: ' + current_date + '\n'
        header += 'Server: My-HTTP-Server\n'
        # The following line adding Content-Type is commeted out - it does
        # not work when there are seeral types of files on the page.
        # header += 'Content-type: ' + self._add_content_type(file_type)+'\n'
        if length > 0:
            header += 'Content-Length: ' + str(length) + '\n'
        header += 'Connection: close\n\n'
        return header


    def _wait_for_connections(self):
        """ Loop waiting for connections """
        while True:
            print "Waiting for a new connection"
            self.socket.listen(10)  # maximum number of connections
            conn, addr = self.socket.accept()
            # conn - socket to client
            # addr - clients address ( IP, port)
            print "Got connection from:", addr
            data = conn.recv(SET_LENGTH)          # receive data from client
            # data_to_parse = bytes.decode(data)  # decode it (for Python 3)
            valid_request, resource = self._validate_http_request(data)
            if valid_request:
                if (resource == '/'):      # default in case no file.
                    file_requested = self.www_dir+'/index.html'
                else:
                    file_requested = self.www_dir + resource
                print "Retrieving web page :", file_requested
                server_response = self._attemp_fileread(file_requested)
                conn.send(server_response)
                print "Closing connection with client"
                conn.close()
            else:
                print "Illegal HTTP request"

    def _validate_http_request(self, request):
        """ Check if request is a valid HTTP request and returns TRUE / FALSE
            and the requested URL. 
        """
        request_line1 = request[:request.find('\n')]
        parsed_request = request_line1.split()
        if len(parsed_request) != 3:
            return False, ''
        request_type = parsed_request[FIRST_ELEMENT]
        request_url = parsed_request[SECOND_ELEMENT]
        request_protocol = parsed_request[THIRD_ELEMENT]
        if request_type != 'GET' or request_protocol != 'HTTP/1.1':
            return False, ''
        return True, request_url


    def _attemp_fileread(self, file_requested):
        # Read file content
        file_type = file_requested[file_requested.rfind('.')+1:]
        try:
            file_handler = open(file_requested, 'rb')
            response_content = file_handler.read()  # read content
            file_handler.close()
            response_headers = self._attach_headers(200,file_type,
                                                    len(response_content))
        except Exception as e:  # File opened/read, generate 404 page
            print "Warning, file not accessible. Issuing " \
                  "response code of 404\n", e
            response_headers = self._attach_headers(404, file_type)
            # Build a primitive 'not-found' HTML page
            response_content =  r"<html><body><p>Error 404: " \
                                r"File not found</p><p>My Python HTTP"
            response_content += r"server </p> </body> </html>"
        # server_response = response_headers.encode() # (if python 3)
        # Add the found file or not-found page
        server_response = response_headers
        server_response += response_content
        return server_response

    def _add_content_type(self, file_type):
        """ Adding the appropriate Content-Type according to the file_type
            parameter. Returns the string of type/subtype pair.
        """
        response_type = ''
        file_type = file_type.lower()
        if file_type == 'jpg' or 'jpeg':
            response_type = 'images/jpeg'
        elif file_type == 'txt' or 'html' or 'htm':
            response_type = 'text/html'
        elif file_type == 'js':
            response_type = 'text/javascript'
        elif file_type == 'css':
            response_type = 'text/css'
        else:
            response_type = 'unknown'
        print "response type=", response_type # debug
        # response_type = 'text/html'
        return response_type


def main():
    global myserver
    print "Starting web server"
    myserver = Server(80)       # construct server object
    myserver.activate_server()  # open the socket
if __name__ == '__main__':
    main()